# postacka metrika na metry = alat*difflat+alon*difflon
$alat = 111194.8500267
$alon = 71413.161362

class Point < ApplicationRecord
  belongs_to :neighbour, class_name: 'Point', foreign_key: :near_id, optional: true
#  has_one :chief_neighbour, class_name: 'Point', foreign_key: :near_id
  # lat, lon, name, typ, src, items_src, near_id, cache_popup
  def location
    [lat,lon]
  end
  def near(diameter=50)
    Point.
      where(lat: lat-diameter/$alat..lat+diameter/$alat).
      where(lon: lon-diameter/$alon..lon+diameter/$alon).
      where.not(typ: self.typ)
  end

  def src_hash; JSON.parse(src) end
  def items_array; items_src ? JSON.parse(items_src) : [] end

  def tags; src_hash['properties'] || src_hash['tags'] || [] end

  def color
    return '29BC29' if has_neighbour?
    case typ
      when 'ipr' then 'EB8633' 
      when 'osm' then '1F8D8D'
      else  'EB8633'
    end
  end

  def marker
    return nil if has_neighbour?
    case typ
      when 'ipr' then 'i' # can be a lowercase letter, number or anything from $icon_types
      when 'osm' then 'o'
      when 'seznam' then 's'
      else 'marker'
    end
  end

  def icon_options
    result = $icon_options_base.clone
    result[:icon_url]="/assets/pin-#{marker}.png"
    # result[:icon_url]="https://api.tiles.mapbox.com/v3/marker/pin-m#{marker ? '-'+marker : ''}+#{color}.png"
    return result
  end

  def has_neighbour?; !self.near_id.nil? end

  def reccomended_osm_tags
    return nil unless typ=='ipr'
    {
      'amenity': 'recycling',
      'recycling_type': 'container',
      'operator': 'Pražské služby a.s.',
      'contact:phone': 288409188,
      'contact:website': 'www.psas.cz',
      'ipr:name': name,
      'ref': stationnumber,
      'ref:ipr': tags['ID']
  }.merge(waste_types_reccomended_osm_tags)
  end

  def stationnumber
    sn = tags['STATIONNUMBER']
    raise "Divný typ STATIONNUMBER #{sn}" unless sn =~ /^0*(\d+)\/\s*(EXT)?0*(\d+)$/
    return "#{$1}/#{$2}#{$3}"
  end

  def waste_types
    items_array.map do |item|
      item['properties']['TRASHTYPENAME']
    end
  end

  def waste_types_reccomended_osm_tags
    rec = []
    waste_types.each do |type|
      case type
        when 'Nápojové kartóny' then rec << 'beverage_cartons'
        when 'Papír' then rec.push('paper', 'newspaper', 'paper_packaging')
        when 'Plast' then rec.push('PET','plastic','polystyrene_foam')
        when 'Kovy' then rec << 'scrap_metal'
        when 'Elektrozařízení' then rec.push('small_electrical_appliances', 'batteries')
        when 'Čiré sklo', 'Barevné sklo'
          # nothing
        else raise "Neznámý typ odpadu #{type}"
      end
    end
    unhandled_glass = ['Čiré sklo', 'Barevné sklo' ] - waste_types
    if unhandled_glass.empty? or unhandled_glass==['Čiré sklo'] 
      rec << 'glass'
    elsif unhandled_glass.size==1
      raise "Jen jeden typ skla v #{waste_types}"
    end
    return rec.map { |r| ["recycling:#{r}",'yes'] }.to_h
  end

  def tags_inspect
    tags.to_yaml.gsub!(': ', '=').gsub("---\n",'')
  end

  def to_s; name end 

  def osmid; src_hash['id'] end

  def sid
    stags = Nokogiri.XML(src).css('extensions').first.element_children.map{|el| [el.name,el.content]}.to_h
    stags['ID'].to_i
  end

end 

# http://paletton.com/#uid=72P0u0kp5tyfrFkk-v+tepgvzk2
